package model;

public class ManModel {

    public final static int STATE_IDLE = -1;
    public final static int STATE_MOVING_LEFT = 0;
    public final static int STATE_MOVING_RIGHT = 1;
    public final static int STATE_JUMPING = 2; // jump, but not simply falling
    public final static int STATE_FALLING = 3; // just fall  from a platform
    public final static int STATE_COLLIDE = 4; // just stop
    public static int MOVE_LEFT = -1;
    public static int MOVE_RIGHT = 1;
    public static int JUMP_LEFT = -1;
    public static int JUMP_UP = 0;
    public static int JUMP_RIGHT = 1;

    private final static double gravity = 2; // used to manage speedY when jumping/falling

    private final double baseSpeed = 6;
    private double speedX;
    private double speedY;
    // position in absolute space (NB: scene space = absolute space for convenience)
    private double x;
    private double y;
    // position when a collision occurred: computed by the controller
    private double collisionX;
    private double collisionY;
    // state
    private int state;
    // flags for different events
    private boolean positionChanged;
    private boolean collisionOccurred;
    private boolean endCollisionOccurred;
    // counter associated to flags
    private int counterCollision;

    public ManModel() {
        speedX = 0.0;
        speedY = 0.0;
        x = 0;
        y = 0;
        collisionX = 0.0;
        collisionY = 0.0;
        positionChanged = false;
        collisionOccurred = false;
        endCollisionOccurred = false;
        counterCollision = 0;
        state = STATE_IDLE;
    }

    public double getX() {
        return x;
    }

    public double getY() {
        return y;
    }


    public boolean isPositionChanged() {
        return positionChanged;
    }

    public void resetFlags() {
        positionChanged = false;
        collisionOccurred = false;
        endCollisionOccurred = false;
    }

    public int getState() {
        return state;
    }

    public void setStateCollision(double collisionX, double collisionY) {
        state = STATE_COLLIDE;
        this.collisionX = collisionX;
        this.collisionY = collisionY;
        collisionOccurred = true;
        counterCollision = 10;
    }

    public boolean isCollisionOccurred() {
        return collisionOccurred;
    }

    public boolean isEndCollisionOccurred() {
        return endCollisionOccurred;
    }

    public void put(double x, double y) {
        if ((this.x != x) || (this.y != y)) {
            positionChanged = true;
            this.x = x;
            this.y = y;
        }
    }

    public void setStateMoveLeft() {
        state = STATE_MOVING_LEFT;
        speedX = -baseSpeed;
    }

    public void setStateMoveRight() {
        state = STATE_MOVING_RIGHT;
        speedX = baseSpeed;
    }

    private void move() {
        if ((state == STATE_MOVING_LEFT) || (state == STATE_MOVING_RIGHT)) {
            x += speedX;
            positionChanged = true;
            stop();
        }
    }

    public void setStateJump(int direction) {
        // NB : can only do a jump when not jumping or falling
        //System.out.println("do jump");
        state = STATE_JUMPING;
        if (direction == JUMP_RIGHT) {
            speedX = baseSpeed* 1.5;
        }
        else if (direction == JUMP_LEFT) {
            speedX = -baseSpeed* 1.5;
        }
        else {
            speedX = 0.0;
        }
        speedY = -25;
    }

    private void jump() {
        if (state == STATE_JUMPING) {
            x += speedX;
            speedY += gravity;
            y += speedY;
            positionChanged = true;
        }
    }

    public void setStateFall() {
        state = STATE_FALLING;
        speedX = 0.0;
        speedY = 0.0;

    }
    private void fall() {
        if (state == STATE_FALLING) {
            speedY += gravity;
            y += speedY;
            positionChanged = true;
        }
    }

    public double[] getPosition() {
        double[] tab = {getX(), getY()};
        return tab;
    }

    public double[] getNextPosition() {
        if (state == STATE_MOVING_LEFT) {
            double[] tab = {x-baseSpeed,y};
            return tab;
        }
        else if (state == STATE_MOVING_RIGHT) {
            double[] tab = {x+baseSpeed,y};
            return tab;
        }
        else if ((state == STATE_FALLING) || (state == STATE_JUMPING)) {
            double[] tab = {x+speedX, y+speedY+gravity};
            return tab;
        }

        double[] tab = {getX(), getY()};
        return tab;
    }

    public void stop() {

        speedX = 0.0;
        speedY = 0.0;
        state = STATE_IDLE;
    }

    public void update() {

        counterCollision--;
        if ( counterCollision == 0) {
            endCollisionOccurred = true;
        }

        if (state == STATE_COLLIDE) {
            // controller already computed the new position
            // thus just signal that to the view
            stop();
            put(collisionX, collisionY);
            positionChanged = true;
        }
        else if (state == STATE_FALLING) {
            fall();
        }
        else if (state == STATE_JUMPING) {
            jump();
        }
        else if ((state == STATE_MOVING_LEFT) || (state == STATE_MOVING_RIGHT)) {
            move();
            // NB: note that this is a temporary state and move() calls stop() to return to idle state.
        }
    }
}
