package view;

import javafx.geometry.Bounds;
import javafx.scene.Group;
import javafx.scene.Node;
import javafx.scene.paint.Color;
import javafx.scene.shape.Circle;
import javafx.scene.shape.Shape;
import model.ManModel;

public class ManLook {
    private Group body;
    // shapes that constitute the body
    private Circle outer;
    private Circle inner;
    private double radius;

    private Group hitBox;
    // shape that constitute the hitBox
    private Circle hitCircle;

    private ManModel manModel;

    public ManLook(ManModel manModel, double radius) {

        this.radius = radius;
        this.manModel = manModel;
        // define the body
        body = new Group();
        outer = new Circle();
        outer.setRadius(radius);
        outer.setFill(Color.BLACK);
        outer.setCenterX(radius);
        outer.setCenterY(radius);
        inner = new Circle();
        inner.setRadius(5);
        inner.setFill(Color.WHITE);
        inner.setCenterX(radius);
        inner.setCenterY(radius);
        body.getChildren().addAll(outer, inner);
        // define the hitbox : in the present case, a circle that matches outer boundaries.
        hitBox = new Group();
        hitCircle = new Circle();
        hitCircle.setRadius(radius);
        hitCircle.setFill(Color.TRANSPARENT);
        hitCircle.setCenterX(radius);
        hitCircle.setCenterY(radius);
        hitBox.getChildren().addAll(hitCircle);
    }

    public Group getBody() {
        return body;
    }

    public void setHitBoxPosition(double x, double y) {
        hitBox.setTranslateX(Math.floor(x));
        hitBox.setTranslateY(Math.floor(y));
    }

    // simple case: the hitbox group contains only 1 shape that defines the hitbox
    public Shape getHitBox() {
        return (Shape)(hitBox.getChildren().get(0));
    }

    public void update() {
        if (manModel.isPositionChanged()) {
            body.setTranslateX(Math.floor(manModel.getX()));
            body.setTranslateY(Math.floor(manModel.getY()));
            hitBox.setTranslateX(Math.floor(manModel.getX()));
            hitBox.setTranslateY(Math.floor(manModel.getY()));
        }
        if (manModel.isCollisionOccurred()) {
            inner.setFill(Color.RED);
        }
        if (manModel.isEndCollisionOccurred()) {
            inner.setFill(Color.WHITE);
        }
    }

    public boolean checkCollision(Shape s) {
        for(Node n : hitBox.getChildren()) {
            Shape box = (Shape)n;
            Shape interShape = Shape.intersect(box, s);
            Bounds b = interShape.getBoundsInParent();
            if (b.getWidth() != -1) {
                return true;
            }
        }
        return false;
    }

    public int checkQuadrantCollision(Shape s) {
        double x  = hitBox.getTranslateX();
        double y  = hitBox.getTranslateY();
        Shape interShape = Shape.intersect(getHitBox(), s);
        Bounds b = interShape.getBoundsInParent();
        int collide = 0;
        if (b.getWidth() != -1) {
            if ((b.getMinY() >= y) && ( b.getMinY() <= y+radius)) {
                if ((b.getMinX() >= x) && (b.getMinX() <= x+radius)) {
                    collide += 2;
                }
                if ((b.getMaxX() >= x+radius) && (b.getMaxX() <= x+2.0*radius)) {
                    collide += 1;
                }
            }
            if ((b.getMaxY() >= y+radius) && ( b.getMaxY() <= y+2.0*radius)) {
                if ((b.getMinX() >= x) && (b.getMinX() <= x+radius)) {
                    collide += 4;
                }
                if ((b.getMaxX() >= x+radius) && (b.getMaxX() <= x+2.0*radius)) {
                    collide += 8;
                }
            }
        }
        return collide;
    }
}
