package view;

import javafx.scene.layout.*;
import javafx.scene.shape.Rectangle;
import model.Model;

import java.util.ArrayList;
import java.util.List;

public class View {

    private Model model;
    private Pane rootPane;

    // window related attributes
    private int width;
    private int height;
    // look
    private ManLook manLook;
    private List<Rectangle> obstacles;

    public View(Model model) {

        this.model = model;
        width = model.getWidth();
        height = model.getHeight();

        obstacles = new ArrayList<>();

        manLook = new ManLook(model.getMan(), 20);

        // create north wall
        obstacles.add(new Rectangle(0,0,width, 10));
        // create south wall
        obstacles.add(new Rectangle(0,height-10,width, 10));
        // create west wall
        obstacles.add(new Rectangle(0,0,10,height));
        // create east wall
        obstacles.add(new Rectangle(width-10,0,10,height));
        // create platforms
        Rectangle rect = new Rectangle(100, height-100, 100, 10);
        // NB: you can uncomment the following line and test different orientations (e.g. 1, 5, 10, 25, 45) to see how the Man behaves
        // but whatever the case, the Man can never go freely to the left, all along this rectangle: it will be blocked as soon as
        // there is a pixel the collides with the very base of the Man.
        rect.setRotate(5);
        obstacles.add(rect);
        obstacles.add(new Rectangle(250, height-200, 100, 10));
        rootPane = new Pane();
        rootPane.setPrefSize(width,height);

        rootPane.getChildren().add(manLook.getBody());
        for(Rectangle r : obstacles) {
            rootPane.getChildren().add(r);
        }
    }

    public int getWidth() {
        return width;
    }

    public int getHeight() {
        return height;
    }

    public Pane getRootPane() {
        return rootPane;
    }

    public ManLook getManLook() {
        return manLook;
    }

    public List<Rectangle> getObstacles() {
        return obstacles;
    }

    public void update() {
        manLook.update();
    }
}
